@@ -8,28 +8,31 @@ BEGIN {
 
     require Exporter;
     use vars    qw[$VERSION @ISA @EXPORT];
-  
-    $VERSION    = '0.41_01';
+
+    $VERSION    = '0.61_01';
     @ISA        = qw[Exporter];
-    @EXPORT     = qw[ BUILD_DIR BUILD ];
+    @EXPORT     = qw[ BUILD_DIR BUILD CPDB_PERL_WRAPPER];
 }
 
 
 use constant BUILD_DIR      => sub { return @_
                                         ? File::Spec->catdir($_[0], '_build')
                                         : '_build';
-                            }; 
+                            };
 use constant BUILD          => sub { my $file = @_
                                         ? File::Spec->catfile($_[0], 'Build')
                                         : 'Build';
-                                        
+
                                      ### on VMS, '.com' is appended when
                                      ### creating the Build file
-                                     $file .= '.com' if $^O eq 'VMS';     
-                                     
+                                     $file .= '.com' if $^O eq 'VMS';
+
                                      return $file;
                             };
-                            
+
+
+use constant CPDB_PERL_WRAPPER   => 'use strict; BEGIN { my $old = select STDERR; $|++; select $old; $|++; $0 = shift(@ARGV); my $rv = do($0); die $@ if $@; }';
+
 1;
 
 =head1 NAME
